@@ -3,10 +3,13 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include "accountmanager.h"
+#include "account.h"
+#include "networkaccessmanagerfactory.h"
 
 AccountManager::AccountManager(QObject *parent)
     : QAbstractListModel(parent)
     , m_selected_account(nullptr)
+    , m_qnam(NetworkAccessManagerFactory().create(this))
 {
     QSettings settings;
     loadFromSettings(settings);
@@ -18,8 +21,9 @@ AccountManager::~AccountManager()
     QSettings settings;
     writeToSettings(settings);
 
-    for (auto a : m_accounts)
+    for (auto a : std::as_const(m_accounts)) {
         delete a;
+    }
 
     m_accounts.clear();
 }
@@ -30,17 +34,23 @@ QVariant AccountManager::data(const QModelIndex &index, int role) const
         return {};
     }
 
+    auto account = m_accounts.at(index.row());
     switch (role) {
-        case Qt::DisplayRole:
-            return m_accounts[index.row()]->username();
-        case AccountRole:
-            return QVariant::fromValue(m_accounts[index.row()]);
+    case Qt::DisplayRole:
+        return account->identity().displayNameHtml();
+    case DescriptionRole:
+        return account->identity().account();
+    case InstanceRole:
+        return account->instanceName();
+    case AccountRole:
+        return QVariant::fromValue(m_accounts[index.row()]);
     }
     return {};
 }
 
 int AccountManager::rowCount(const QModelIndex &index) const
 {
+    Q_UNUSED(index)
     return m_accounts.size();
 }
 
@@ -48,7 +58,9 @@ QHash<int, QByteArray> AccountManager::roleNames() const
 {
     return {
         {Qt::DisplayRole, QByteArrayLiteral("display")},
-        {AccountRole, QByteArrayLiteral("account")}
+        {AccountRole, QByteArrayLiteral("account")},
+        {DescriptionRole, QByteArrayLiteral("description")},
+        {InstanceRole, QByteArrayLiteral("instance")},
     };
 }
 
@@ -58,9 +70,9 @@ AccountManager &AccountManager::instance()
     return accountManager;
 }
 
-Account *AccountManager::createNewAccount(const QString &username, const QString &instanceUri)
+AbstractAccount *AccountManager::createNewAccount(const QString &username, const QString &instanceUri, bool ignoreSslErrors)
 {
-    return new Account(username, instanceUri, this);
+    return new Account(username, instanceUri, m_qnam, ignoreSslErrors, this);
 }
 
 bool AccountManager::hasAccounts() const
@@ -68,7 +80,7 @@ bool AccountManager::hasAccounts() const
     return m_accounts.size() > 0;
 }
 
-void AccountManager::addAccount(Account *account)
+void AccountManager::addAccount(AbstractAccount *account)
 {
     beginInsertRows(QModelIndex(), m_accounts.size(), m_accounts.size());
     m_accounts.append(account);
@@ -77,43 +89,52 @@ void AccountManager::addAccount(Account *account)
     Q_EMIT accountAdded(account);
     selectAccount(account);
 
-    QObject::connect(account, &Account::identityChanged, this, &AccountManager::childIdentityChanged);
+    connect(account, &Account::identityChanged, this, &AccountManager::childIdentityChanged);
+    connect(account, &Account::authenticated, this, [this]() {
+        Q_EMIT dataChanged(index(0, 0), index(m_accounts.size() - 1, 0));
+    });
 
-    QObject::connect(account, &Account::fetchedTimeline, [=] (QString original_name, QList<std::shared_ptr<Post>> posts) {
+    connect(account, &Account::fetchedTimeline, this, [this, account](QString original_name, QList<std::shared_ptr<Post>> posts) {
         Q_EMIT fetchedTimeline(account, original_name, posts);
     });
-    QObject::connect(account, &Account::invalidated, [=] () {
+    connect(account, &Account::invalidated, this, [this, account]() {
         Q_EMIT invalidated(account);
     });
-    QObject::connect(account, &Account::fetchedInstanceMetadata, [=] () {
+    connect(account, &Account::fetchedInstanceMetadata, this, [this, account]() {
         Q_EMIT fetchedInstanceMetadata(account);
     });
-    QObject::connect(account, &Account::invalidatedPost, [=] (Post *p) {
+    connect(account, &Account::invalidatedPost, this, [this, account](Post *p) {
         Q_EMIT invalidatedPost(account, p);
     });
-    QObject::connect(account, &Account::notification, [=] (std::shared_ptr<Notification> n) {
+    connect(account, &Account::notification, this, [this, account](std::shared_ptr<Notification> n) {
         Q_EMIT notification(account, n);
     });
     QSettings settings;
     writeToSettings(settings);
 }
 
-void AccountManager::childIdentityChanged(Account *account)
+void AccountManager::childIdentityChanged(AbstractAccount *account)
 {
     Q_EMIT identityChanged(account);
 }
 
-void AccountManager::removeAccount(Account *account)
+void AccountManager::removeAccount(AbstractAccount *account)
 {
+    const auto index = m_accounts.indexOf(account);
+    beginRemoveRows(QModelIndex(), index, index);
     m_accounts.removeOne(account);
+    endRemoveRows();
+
+    QSettings settings;
+    settings.clear();
+    writeToSettings(settings);
 
     Q_EMIT accountRemoved(account);
 }
 
-void AccountManager::selectAccount(Account *account)
+void AccountManager::selectAccount(AbstractAccount *account)
 {
-    if (!m_accounts.contains(account))
-    {
+    if (!m_accounts.contains(account)) {
         qDebug() << "WTF: attempt to select unmanaged account" << account;
         return;
     }
@@ -123,17 +144,28 @@ void AccountManager::selectAccount(Account *account)
     Q_EMIT accountSelected(account);
 }
 
-Account *AccountManager::selectedAccount()
+AbstractAccount *AccountManager::selectedAccount() const
 {
     return m_selected_account;
 }
 
+int AccountManager::selectedIndex() const
+{
+    for (int i = 0; i < m_accounts.length(); i++) {
+        if (m_selected_account == m_accounts[i]) {
+            return i;
+        }
+    }
+    return -1;
+}
+
 void AccountManager::writeToSettings(QSettings &settings)
 {
     settings.beginGroup("accounts");
 
-    for (auto a : m_accounts)
+    for (auto a : std::as_const(m_accounts)) {
         a->writeToSettings(settings);
+    }
 
     settings.endGroup();
 }
@@ -148,7 +180,7 @@ void AccountManager::loadFromSettings(QSettings &settings)
     for (const auto &child : childGroups) {
         settings.beginGroup(child);
 
-        auto account = new Account(settings);
+        auto account = new Account(settings, m_qnam);
         if (account->haveToken()) {
             addAccount(account);
             qDebug() << "Loaded from settings:" << account;
