@@ -4,38 +4,19 @@
 
 #pragma once
 
-#include "post.h"
+#include "abstracttimelinemodel.h"
 #include "accountmanager.h"
+#include "post.h"
 #include <QAbstractListModel>
 
-class NotificationModel : public QAbstractListModel
+class NotificationModel : public AbstractTimelineModel
 {
     Q_OBJECT
     Q_PROPERTY(QStringList excludeTypes READ excludeTypes WRITE setExcludesTypes NOTIFY excludeTypesChanged)
 
 public:
-    enum CustoRoles {
-        AvatarRole = Qt::UserRole +1,
-        AuthorDisplayNameRole,
-        PublishedAtRole,
-        AuthorIdRole,
-        RelativeTimeRole,
-        SensitiveRole,
-        SpoilerTextRole,
-        MutedRole,
-        PinnedRole,
-        AttachmentsRole,
-        WasRebloggedRole,
-        RebloggedDisplayNameRole,
-        RebloggedIdRole,
-        RebloggedRole,
-        ReblogsCountRole,
-        RepliesCountRole,
-        FavoritedRole,
-        FavoritesCountRole,
-        ThreadModelRole,
-        AccountModelRole,
-        UrlRole,
+    enum ExtraRoles {
+        NewFollowerIdentityRole = AbstractTimelineModel::ExtraRole,
     };
 
     explicit NotificationModel(QObject *parent = nullptr);
@@ -43,10 +24,11 @@ public:
     QVariant data(const QModelIndex &index, int role) const override;
     QHash<int, QByteArray> roleNames() const override;
 
-    virtual void fillTimeline(const QString &fromId = QString());
+    virtual void fillTimeline(const QUrl &next = {});
 
-    void disallowUpdates() { m_last_fetch = time(nullptr) + 3; }
-    std::shared_ptr<Post> internalData(const QModelIndex &index) const;
+    std::shared_ptr<Notification> internalData(const QModelIndex &index) const;
+    QStringList excludeTypes() const;
+    void setExcludesTypes(const QStringList &excludeTypes);
 
 public Q_SLOTS:
     void actionReply(const QModelIndex &index);
@@ -57,21 +39,22 @@ public Q_SLOTS:
 
 Q_SIGNALS:
     void excludeTypesChanged();
+    void wantReply(AbstractAccount *account, std::shared_ptr<Post> post, const QModelIndex &index);
+    void wantMenu(AbstractAccount *account, std::shared_ptr<Post> post, const QModelIndex &index);
 
 protected:
     void fetchMore(const QModelIndex &parent) override;
     bool canFetchMore(const QModelIndex &parent) const override;
 
-    QString m_timeline_name;
+    QString m_timelineName;
     AccountManager *m_manager = nullptr;
-    Account *m_account = nullptr;
+    AbstractAccount *m_account = nullptr;
 
     QList<std::shared_ptr<Notification>> m_notifications;
     bool m_fetching;
-    time_t m_last_fetch;
     QStringList m_excludeTypes;
+    QUrl m_next;
 
 public Q_SLOTS:
     void fetchedNotifications(QList<std::shared_ptr<Notification>> notifications);
 };
-
