@@ -5,18 +5,59 @@
 #pragma once
 
 #include <QDateTime>
-#include <QUrl>
-#include <QString>
-#include <QJsonObject>
 #include <QImage>
-#include <QObject>
+#include <QJsonObject>
 #include <QNetworkReply>
+#include <QObject>
+#include <QString>
+#include <QUrl>
 
+#include <QJsonObject>
 #include <memory>
+#include <optional>
 
-struct Post;
-struct Identity;
-class Account;
+class Post;
+class Identity;
+class AbstractAccount;
+
+class Card
+{
+    Q_GADGET
+    Q_PROPERTY(QString authorName READ authorName)
+    Q_PROPERTY(QString authorUrl READ authorUrl)
+    Q_PROPERTY(QString blurhash READ blurhash)
+    Q_PROPERTY(QString description READ description)
+    Q_PROPERTY(QString embedUrl READ embedUrl)
+    Q_PROPERTY(int width READ width)
+    Q_PROPERTY(int height READ height)
+    Q_PROPERTY(QString html READ html)
+    Q_PROPERTY(QString image READ image)
+    Q_PROPERTY(QString providerName READ providerName)
+    Q_PROPERTY(QString providerUrl READ providerUrl)
+    Q_PROPERTY(QString title READ title)
+    Q_PROPERTY(QUrl url READ url)
+
+public:
+    Card() = default;
+    Card(QJsonObject card);
+
+    QString authorName() const;
+    QString authorUrl() const;
+    QString blurhash() const;
+    QString description() const;
+    QString embedUrl() const;
+    int width() const;
+    int height() const;
+    QString html() const;
+    QString image() const;
+    QString providerName() const;
+    QString providerUrl() const;
+    QString title() const;
+    QUrl url() const;
+
+private:
+    QJsonObject m_card;
+};
 
 /// Post's attachment object.
 /// TODO make it possible to fetch the images with a Qml image provider.
@@ -30,17 +71,13 @@ class Attachment : public QObject
     Q_PROPERTY(QString previewUrl MEMBER m_preview_url CONSTANT)
     Q_PROPERTY(QString url MEMBER m_url CONSTANT)
     Q_PROPERTY(QString description MEMBER m_description CONSTANT)
+    Q_PROPERTY(QString blurhash MEMBER m_blurhash CONSTANT)
 
 public:
-    Attachment(Post *parent, QJsonObject &obj);
+    explicit Attachment(Post *parent, const QJsonObject &obj);
     ~Attachment();
 
-    enum AttachmentType {
-        Unknown,
-        Image,
-        GifV,
-        Video
-    };
+    enum AttachmentType { Unknown, Image, GifV, Video };
     Post *m_parent;
 
     QString m_id;
@@ -48,32 +85,47 @@ public:
     QString m_preview_url;
     QString m_url;
     QString m_description;
+    QString m_blurhash;
 
-    void setDescription(QString desc);
+    void setDescription(const QString &description);
 };
 
-struct Notification
+class Notification
 {
-    Notification() = delete;
-    Notification(const Notification &) = delete;
-    Notification(Account *parent, QJsonObject &obj);
+    Q_GADGET
+
+public:
+    Notification()
+    {
+    }
+    Notification(const Notification &)
+    {
+    }
+    Notification(AbstractAccount *parent, const QJsonObject &obj);
 
     enum Type {
         Mention,
         Follow,
         Repeat,
         Favorite,
+        Poll,
+        FollowRequest,
     };
+    Q_ENUM(Type);
 
-    Account *m_account;
-    Type m_type;
-
-    std::shared_ptr<Post> m_post;
-    std::shared_ptr<Identity> m_identity;
     int id() const;
+    AbstractAccount *account() const;
+    Type type() const;
+    std::shared_ptr<Post> post() const;
+    std::shared_ptr<Identity> identity() const;
 
 private:
     int m_id;
+
+    AbstractAccount *m_account;
+    Type m_type;
+    std::shared_ptr<Post> m_post;
+    std::shared_ptr<Identity> m_identity;
 };
 
 class Post : public QObject
@@ -90,8 +142,8 @@ class Post : public QObject
 public:
     Post() = delete;
     Post(const Post &) = delete;
-    Post(Account *parent);
-    Post(Account *parent, QJsonObject obj);
+    Post(AbstractAccount *parent);
+    Post(AbstractAccount *parent, QJsonObject obj);
     ~Post();
 
     enum Visibility {
@@ -102,9 +154,10 @@ public:
     };
     Q_ENUM(Visibility)
 
-    Account *m_parent;
-    std::shared_ptr<Identity> m_author_identity;
-    std::shared_ptr<Identity> m_repeat_identity;
+    AbstractAccount *m_parent;
+
+    std::shared_ptr<Identity> authorIdentity() const;
+    std::shared_ptr<Identity> repeatIdentity() const;
 
     QString subject() const;
     void setSubject(const QString &subject);
@@ -120,7 +173,10 @@ public:
     void setInReplyTo(const QString &inReplyTo);
     QStringList mentions() const;
     void setMentions(const QStringList &mentions);
+    std::optional<Card> card() const;
+    void setCard(std::optional<Card> card);
 
+    void addAttachment(const QJsonObject &attachment);
     Q_INVOKABLE void uploadAttachment(const QUrl &filename);
 
     bool m_repeat = false;
@@ -134,6 +190,7 @@ public:
 
     QDateTime m_published_at;
     QString m_post_id;
+    QString m_original_post_id;
     QUrl m_link;
     QString m_content;
     QString m_subject;
@@ -145,8 +202,11 @@ public:
     QStringList m_mentions;
     bool m_pinned = false;
 
-    bool isEmpty() { return m_post_id.isEmpty(); }
-    Q_INVOKABLE void addAttachments(const QJsonArray& attachments);
+    bool isEmpty()
+    {
+        return m_post_id.isEmpty();
+    }
+    Q_INVOKABLE void addAttachments(const QJsonArray &attachments);
     void setDirtyAttachment();
     void updateAttachment(Attachment *a);
 
@@ -161,7 +221,14 @@ Q_SIGNALS:
     void visibilityChanged();
     void inReplyToChanged();
     void mentionsChanged();
+    void attachmentUploaded();
 
 private:
     QString m_replyTargetId;
+    std::optional<Card> m_card;
+    std::shared_ptr<Identity> m_authorIdentity;
+    std::shared_ptr<Identity> m_repeatIdentity;
 };
+
+Q_DECLARE_METATYPE(Card)
+Q_DECLARE_METATYPE(Notification)
