@@ -3,33 +3,41 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include "timelinemodel.h"
+#include "abstractaccount.h"
+#include "abstracttimelinemodel.h"
 #include "accountmodel.h"
+#include "identity.h"
 #include "threadmodel.h"
 #include <KLocalizedString>
 #include <QtMath>
 
 TimelineModel::TimelineModel(QObject *parent)
-    : QAbstractListModel(parent)
+    : AbstractTimelineModel(parent)
+    , m_last_fetch(time(nullptr))
 {
 }
 
-void TimelineModel::setName(const QString &timeline_name)
+void TimelineModel::setName(const QString &timelineName)
 {
-    if (timeline_name == m_timeline_name) {
+    if (timelineName == m_timelineName) {
         return;
     }
 
-    m_timeline_name = timeline_name;
+    m_timelineName = timelineName;
     Q_EMIT nameChanged();
 }
 
 QString TimelineModel::displayName() const
 {
-    if (m_timeline_name == "home") {
-        return i18nc("@title", "Home");
-    } else if (m_timeline_name == "public") {
+    if (m_timelineName == "home") {
+        if (m_manager && m_manager->rowCount() > 1) {
+            return i18nc("@title", "Home (%1)", m_manager->selectedAccount()->username());
+        } else {
+            return i18nc("@title", "Home");
+        }
+    } else if (m_timelineName == "public") {
         return i18nc("@title", "Local Timeline");
-    } else if (m_timeline_name == "federated") {
+    } else if (m_timelineName == "federated") {
         return i18nc("@title", "Global Timeline");
     }
     return QString();
@@ -50,7 +58,7 @@ void TimelineModel::setAccountManager(AccountManager *accountManager)
 
     Q_EMIT accountManagerChanged();
 
-    QObject::connect(m_manager, &AccountManager::accountSelected, this, [=] (Account *account) {
+    QObject::connect(m_manager, &AccountManager::accountSelected, this, [=](AbstractAccount *account) {
         if (m_account == account) {
             return;
         }
@@ -63,7 +71,7 @@ void TimelineModel::setAccountManager(AccountManager *accountManager)
         fillTimeline();
     });
     QObject::connect(m_manager, &AccountManager::fetchedTimeline, this, &TimelineModel::fetchedTimeline);
-    QObject::connect(m_manager, &AccountManager::invalidated, this, [=] (Account *account) {
+    QObject::connect(m_manager, &AccountManager::invalidated, this, [=](AbstractAccount *account) {
         if (m_account == account) {
             qDebug() << "Invalidating account" << account;
 
@@ -85,19 +93,20 @@ AccountManager *TimelineModel::accountManager() const
 
 QString TimelineModel::name() const
 {
-    return m_timeline_name;
+    return m_timelineName;
 }
 
-void TimelineModel::fillTimeline(QString from_id)
+void TimelineModel::fillTimeline(const QString &from_id)
 {
-    if (m_timeline_name != "home" && m_timeline_name != "public" && m_timeline_name != "federated") {
+    if (m_timelineName != "home" && m_timelineName != "public" && m_timelineName != "federated") {
         return;
     }
 
     m_fetching = true;
+    Q_EMIT fetchingChanged();
 
     if (m_account) {
-        m_account->fetchTimeline(m_timeline_name, from_id);
+        m_account->fetchTimeline(m_timelineName, from_id);
     }
 }
 
@@ -110,7 +119,7 @@ void TimelineModel::fetchMore(const QModelIndex &parent)
 
     auto p = m_timeline.last();
 
-    fillTimeline(p->m_post_id);
+    fillTimeline(p->m_original_post_id);
 }
 
 bool TimelineModel::canFetchMore(const QModelIndex &parent) const
@@ -126,44 +135,46 @@ bool TimelineModel::canFetchMore(const QModelIndex &parent) const
     return true;
 }
 
-void TimelineModel::fetchedTimeline(Account *account, QString original_name, QList<std::shared_ptr<Post>> posts)
+void TimelineModel::fetchedTimeline(AbstractAccount *account, const QString &original_name, const QList<std::shared_ptr<Post>> &posts)
 {
     m_fetching = false;
+    Q_EMIT fetchingChanged();
+
+    m_loading = false;
+    Q_EMIT loadingChanged();
 
     // make sure the timeline update is for us
-    if (account != m_account || original_name != m_timeline_name) {
+    if (account != m_account || original_name != m_timelineName) {
         return;
     }
 
-    if (posts.isEmpty())
+    if (posts.isEmpty()) {
         return;
-
-    int row = 0;
-    int last = 0;
+    }
 
     if (!m_timeline.isEmpty()) {
-        auto post_old = m_timeline.first();
-        auto post_new = posts.first();
+        const auto post_old = m_timeline.first();
+        const auto post_new = posts.first();
 
-        qDebug() << "fetchedTimeline" << "post_old->m_post_id" << post_old->m_post_id << "post_new->m_post_id" << post_new->m_post_id;
+        qDebug() << "fetchedTimeline"
+                 << "post_old->m_post_id" << post_old->m_post_id << "post_new->m_post_id" << post_new->m_post_id;
         if (post_old->m_post_id > post_new->m_post_id) {
-            row = m_timeline.size();
-            last = row + posts.size() - 1;
+            const int row = m_timeline.size();
+            const int last = row + posts.size() - 1;
+            beginInsertRows(QModelIndex(), row, last);
             m_timeline += posts;
+            endInsertRows();
         } else {
-            row = 0;
-            last = posts.size();
+            beginInsertRows(QModelIndex(), 0, posts.size() - 1);
             m_timeline = posts + m_timeline;
+            endInsertRows();
         }
     } else {
-        row = 0;
-        last = posts.size() - 1;
+        beginInsertRows(QModelIndex(), 0, posts.size() - 1);
         m_timeline = posts;
+        endInsertRows();
     }
 
-    beginInsertRows(QModelIndex(), row, last);
-    endInsertRows();
-
     m_last_fetch = time(nullptr);
 }
 
@@ -181,35 +192,6 @@ std::shared_ptr<Post> TimelineModel::internalData(const QModelIndex &index) cons
     return m_timeline[row];
 }
 
-QHash<int, QByteArray> TimelineModel::roleNames() const
-{
-    return {
-        {Qt::DisplayRole, QByteArrayLiteral("display")},
-        {AvatarRole, QByteArrayLiteral("avatar")},
-        {AuthorDisplayNameRole, QByteArrayLiteral("authorDisplayName")},
-        {PinnedRole, QByteArrayLiteral("pinned")},
-        {IdRole, QByteArrayLiteral("id")},
-        {AuthorIdRole, QByteArrayLiteral("authorId")},
-        {PublishedAtRole, QByteArrayLiteral("publishedAt")},
-        {MentionsRole, QByteArrayLiteral("mentions")},
-        {RelativeTimeRole, QByteArrayLiteral("relativeTime")},
-        {SensitiveRole, QByteArrayLiteral("sensitive")},
-        {SpoilerTextRole, QByteArrayLiteral("spoilerText")},
-        {RebloggedRole, QByteArrayLiteral("reblogged")},
-        {WasRebloggedRole, QByteArrayLiteral("wasReblogged")},
-        {RebloggedDisplayNameRole, QByteArrayLiteral("rebloggedDisplayName")},
-        {RebloggedIdRole, QByteArrayLiteral("rebloggedId")},
-        {AttachmentsRole, QByteArrayLiteral("attachments")},
-        {ReblogsCountRole, QByteArrayLiteral("reblogsCount")},
-        {RepliesCountRole, QByteArrayLiteral("repliesCount")},
-        {FavoritedRole, QByteArrayLiteral("favorite")},
-        {FavoritesCountRole, QByteArrayLiteral("favoritesCount")},
-        {UrlRole, QByteArrayLiteral("url")},
-        {ThreadModelRole, QByteArrayLiteral("threadModel")},
-        {AccountModelRole, QByteArrayLiteral("accountModel")}
-    };
-}
-
 QVariant TimelineModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid()) {
@@ -219,16 +201,18 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
     auto p = m_timeline[row];
 
     switch (role) {
+    case TypeRole:
+        return false;
     case IdRole:
         return p->m_post_id;
     case Qt::DisplayRole:
         return p->m_content;
     case AvatarRole:
-        return p->m_author_identity->m_avatarUrl;
+        return p->authorIdentity()->avatarUrl();
     case AuthorDisplayNameRole:
-        return p->m_author_identity->m_display_name;
+        return p->authorIdentity()->displayNameHtml();
     case AuthorIdRole:
-        return p->m_author_identity->m_acct;
+        return p->authorIdentity()->account();
     case PublishedAtRole:
         return p->m_published_at;
     case WasRebloggedRole:
@@ -236,13 +220,13 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
     case MentionsRole:
         return p->mentions();
     case RebloggedDisplayNameRole:
-        if (p->m_repeat_identity) {
-            return p->m_repeat_identity->m_display_name;
+        if (p->repeatIdentity()) {
+            return p->repeatIdentity()->displayNameHtml();
         }
         return {};
     case RebloggedIdRole:
-        if (p->m_repeat_identity) {
-            return p->m_repeat_identity->m_acct;
+        if (p->repeatIdentity()) {
+            return p->repeatIdentity()->account();
         }
         return {};
     case RebloggedRole:
@@ -251,6 +235,8 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
         return p->m_repeatedCount;
     case FavoritedRole:
         return p->m_isFavorite;
+    case FavoritesCountRole:
+        return p->m_favoriteCount;
     case PinnedRole:
         return p->m_pinned;
     case SensitiveRole:
@@ -261,24 +247,32 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
         return p->m_subject;
     case AttachmentsRole:
         return QVariant::fromValue<QList<Attachment *>>(p->m_attachments);
+    case CardRole:
+        if (p->card().has_value()) {
+            return QVariant::fromValue<Card>(*p->card());
+        }
+        return false;
     case ThreadModelRole:
         return QVariant::fromValue<QAbstractListModel *>(new ThreadModel(m_manager, p->m_post_id));
     case AccountModelRole:
-        return QVariant::fromValue<QAbstractListModel *>(new AccountModel(m_manager, p->m_author_identity->m_id, p->m_author_identity->m_acct));
-    case RelativeTimeRole:
-        {
-            const auto current = QDateTime::currentDateTime();
-            auto secsTo = p->m_published_at.secsTo(current);
-            if (secsTo < 60 * 60) {
-                return i18nc("hour:minute", "%1:%2", p->m_published_at.time().hour(),
-                        p->m_published_at.time().minute());
-            } else if (secsTo < 60 * 60 * 24) {
-                return i18n("%1h", qCeil(secsTo / (60 * 60)));
-            } else if (secsTo < 60 * 60 * 24 * 7) { 
-                return i18n("%1d", qCeil(secsTo / (60 * 60 * 24)));
-            }
-            return p->m_published_at.date().toString(Qt::SystemLocaleShortDate);
+        return QVariant::fromValue<QAbstractListModel *>(new AccountModel(m_manager, p->authorIdentity()->id(), p->authorIdentity()->account()));
+    case RelativeTimeRole: {
+        const auto current = QDateTime::currentDateTime();
+        auto secsTo = p->m_published_at.secsTo(current);
+        if (secsTo < 60 * 60) {
+            const auto hours = p->m_published_at.time().hour();
+            const auto minutes = p->m_published_at.time().minute();
+            return i18nc("hour:minute",
+                         "%1:%2",
+                         hours < 10 ? QChar('0') + QString::number(hours) : QString::number(hours),
+                         minutes < 10 ? QChar('0') + QString::number(minutes) : QString::number(minutes));
+        } else if (secsTo < 60 * 60 * 24) {
+            return i18n("%1h", qCeil(secsTo / (60 * 60)));
+        } else if (secsTo < 60 * 60 * 24 * 7) {
+            return i18n("%1d", qCeil(secsTo / (60 * 60 * 24)));
         }
+        return QLocale::system().toString(p->m_published_at.date(), QLocale::ShortFormat);
+    }
     }
 
     return {};
@@ -286,26 +280,26 @@ QVariant TimelineModel::data(const QModelIndex &index, int role) const
 
 void TimelineModel::actionReply(const QModelIndex &index)
 {
-    int row = index.row ();
+    int row = index.row();
     auto p = m_timeline[row];
 
-    Q_EMIT wantReply (m_account, p, index);
+    Q_EMIT wantReply(m_account, p, index);
 }
 
 void TimelineModel::actionMenu(const QModelIndex &index)
 {
-    int row = index.row ();
+    int row = index.row();
     auto p = m_timeline[row];
 
-    Q_EMIT wantMenu (m_account, p, index);
+    Q_EMIT wantMenu(m_account, p, index);
 }
 
 void TimelineModel::actionFavorite(const QModelIndex &index)
 {
-    int row = index.row ();
+    int row = index.row();
     auto p = m_timeline[row];
 
-    if (! p->m_isFavorite) {
+    if (!p->m_isFavorite) {
         m_account->favorite(p);
         p->m_isFavorite = true;
     } else {
@@ -313,7 +307,7 @@ void TimelineModel::actionFavorite(const QModelIndex &index)
         p->m_isFavorite = false;
     }
 
-    Q_EMIT dataChanged(index,index);
+    Q_EMIT dataChanged(index, index);
 }
 
 void TimelineModel::actionRepeat(const QModelIndex &index)
@@ -334,10 +328,20 @@ void TimelineModel::actionRepeat(const QModelIndex &index)
 
 void TimelineModel::actionVis(const QModelIndex &index)
 {
-    int row = index.row ();
+    int row = index.row();
     auto p = m_timeline[row];
 
     p->m_attachments_visible ^= true;
 
     Q_EMIT dataChanged(index, index);
 }
+
+void TimelineModel::refresh()
+{
+    fillTimeline();
+}
+
+bool TimelineModel::fetching() const
+{
+    return m_fetching;
+}
