@@ -4,50 +4,25 @@
 
 #pragma once
 
-#include "post.h"
+#include "abstracttimelinemodel.h"
 #include "accountmanager.h"
+#include "post.h"
 #include <QAbstractListModel>
 
-class TimelineModel : public QAbstractListModel
+class TimelineModel : public AbstractTimelineModel
 {
     Q_OBJECT
     Q_PROPERTY(AccountManager *accountManager READ accountManager WRITE setAccountManager NOTIFY accountManagerChanged)
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged)
     Q_PROPERTY(QString displayName READ displayName NOTIFY nameChanged)
+    Q_PROPERTY(bool fetching READ fetching NOTIFY fetchingChanged)
 
 public:
-    enum CustoRoles {
-        AvatarRole = Qt::UserRole +1,
-        IdRole,
-        AuthorDisplayNameRole,
-        PublishedAtRole,
-        AuthorIdRole,
-        RelativeTimeRole,
-        SensitiveRole,
-        SpoilerTextRole,
-        MutedRole,
-        PinnedRole,
-        AttachmentsRole,
-        WasRebloggedRole,
-        RebloggedDisplayNameRole,
-        RebloggedIdRole,
-        RebloggedRole,
-        ReblogsCountRole,
-        MentionsRole,
-        RepliesCountRole,
-        FavoritedRole,
-        FavoritesCountRole,
-        ThreadModelRole,
-        AccountModelRole,
-        UrlRole,
-    };
-
     explicit TimelineModel(QObject *parent = nullptr);
     int rowCount(const QModelIndex &parent) const override;
     QVariant data(const QModelIndex &index, int role) const override;
-    QHash<int, QByteArray> roleNames() const override;
 
-    virtual void fillTimeline(QString fromId = QString());
+    virtual void fillTimeline(const QString &fromId = QString());
 
     AccountManager *accountManager() const;
     void setAccountManager(AccountManager *accountManager);
@@ -57,9 +32,15 @@ public:
 
     virtual QString displayName() const;
 
-    void disallowUpdates() { m_last_fetch = time(nullptr) + 3; }
+    void disallowUpdates()
+    {
+        m_last_fetch = time(nullptr) + 3;
+    }
     std::shared_ptr<Post> internalData(const QModelIndex &index) const;
 
+    Q_INVOKABLE void refresh();
+
+    bool fetching() const;
 
 public Q_SLOTS:
     void actionReply(const QModelIndex &index);
@@ -69,24 +50,23 @@ public Q_SLOTS:
     void actionVis(const QModelIndex &index);
 
 Q_SIGNALS:
-    void wantReply(Account *account, std::shared_ptr<Post> post, const QModelIndex &index);
-    void wantMenu(Account *account, std::shared_ptr<Post> post, const QModelIndex &index);
+    void wantReply(AbstractAccount *account, std::shared_ptr<Post> post, const QModelIndex &index);
+    void wantMenu(AbstractAccount *account, std::shared_ptr<Post> post, const QModelIndex &index);
     void accountManagerChanged();
     void nameChanged();
+    void fetchingChanged();
 
 protected:
     void fetchMore(const QModelIndex &parent) override;
     bool canFetchMore(const QModelIndex &parent) const override;
 
-    QString m_timeline_name;
+    QString m_timelineName;
     AccountManager *m_manager = nullptr;
-    Account *m_account = nullptr;
 
     QList<std::shared_ptr<Post>> m_timeline;
     bool m_fetching = false;
     time_t m_last_fetch;
 
 public Q_SLOTS:
-    void fetchedTimeline(Account *account, QString original_name, QList<std::shared_ptr<Post>> posts);
+    void fetchedTimeline(AbstractAccount *account, const QString &original_name, const QList<std::shared_ptr<Post>> &posts);
 };
-
